﻿Type=Class
Version=3.82
B4A=true
@EndOfDesignText@
'Class module
'This module plots one curve on a diagram with scales.
'A Text scaling is included that increases the text dimensions according to the screen size.
'The TextAutoScale property allows to enable or disable a text size it. 
'The TextAutoScaleRate property allows to adapt the text size increase.
Sub Class_Globals
	Private cCallBack As Object
	Private cEventName As String
	Private pnlGraph, pnlGrid, pnlCurves  As Panel
	Private cvsGrid, cvsCurves As Canvas
	Private rectCurves, rectPoint As Rect
	Private cLeft, cTop, cWidth, cHeight As Int
	
	Private XGridNb = 10 As Int
	Private YGridNb = 10 As Int
	Private ScaleX, ScaleY As Double
	Private ScaleXMin = -1 As Double
	Private ScaleXMax = 1 As Double
	Private Scale_dX As Double
	Private ScaleYMin = -1 As Double
	Private ScaleYMax = 1 As Double
	Private Scale_dY As Double
	Private XMax, XMin, YMax, YMin As Double
	Private X(), Y() As Double
	Private xx, yy As Int
	Private PlotTitle As String
	Private TextSizeTitle0 = 16 As Float
	Private TextSizeTitle = TextSizeTitle0 As Float
	Private TextSizeCurve0 = 14 As Float
	Private TextSizeCurve = TextSizeCurve0 As Float
	Private TextSizeScale0 = 14 As Float
	Private TextSizeScale = TextSizeScale0 As Float
	
	Private colTitleText = Colors.Black As Int
	Private ScaleTextAuto = True As Boolean
	Private ScaleTextRate = 0.3 As Float
	Private txtScale As Float
	Private GridX0, GridX1, Grid_dX, GridW, GridY0, GridY1, Grid_dY, GridH As Int
	
	Public SamplesNumberMax = 200 As Int
	Private SampleIndex = -1 As Int
	
	Private GridColor = Colors.Gray As Int
	Private GridStroke = 1 As Float
	
	Private colCurveX = Colors.Black As Int
	Private colCurveY = Colors.Blue As Int
	Private StrokeCurve = 2dip As Float
	Private CurveXTitle = "" As String
	Private CurveYTitle = "" As String
	Private ly As LayoutValues
End Sub

'Initiatizes the Plot CustomView
'Callback = calling object, calling module name
'EventName = name for events in the cailling module
Public Sub Initialize(Callback As Object, EventName As String)
	cCallBack = Callback
	cEventName = EventName
End Sub

'Routine for the Designer
Public Sub DesignerCreateView(base As Panel, lbl As Label, props As Map)
	pnlGraph = base
	cLeft = pnlGraph.Left
	cTop = pnlGraph.Top
	cWidth = pnlGraph.Width
	cHeight = pnlGraph.Height
	InitGraph
End Sub

'Adds a Plot CustomView in the code-
'Parent = parent object either an Activity or a Panel
'Left, Top, Widht and Height position and dimensions of the view
Public Sub Add(Parent As Panel, Left As Int, Top As Int, Width As Int, Height As Int)
	pnlGraph.Initialize("pnlGraph")
	
	cLeft = Left
	cTop = Top
	cWidth = Width
	cHeight = Height

	Parent.AddView(pnlGraph, cLeft, cTop, cWidth, cHeight)
	InitGraph
End Sub

'Clear the diagram
Public Sub DrawClear
	cvsCurves.DrawRect(rectCurves, Colors.Transparent, True, 1)
	pnlCurves.Invalidate
End Sub

' Draw a new point onto the chart
Public Sub DrawNewPoint(ValY As Double)
	Dim x1, y1 As Int
	
	SampleIndex = SampleIndex + 1
	If SampleIndex > SamplesNumberMax - 1 OR SampleIndex = 0 Then
		DrawClear
		xx = 0
		x1 = 0
		y1 = (ScaleYMax - ValY) * ScaleY
		If SampleIndex = 0 Then
			yy = y1
		End If
		SampleIndex = 1
	Else
	End If
	x1 = SampleIndex * ScaleX
	y1 = (ScaleYMax - ValY) * ScaleY
	rectPoint.Initialize(xx - StrokeCurve, Min(yy, y1) - StrokeCurve, x1 + StrokeCurve, Max(yy, y1) + StrokeCurve)
	cvsCurves.DrawLine(xx, yy, x1, y1, colCurveY, StrokeCurve)
	xx = x1
	yy = y1
'	pnlCurves.Invalidate2(rectPoint)
	pnlCurves.Invalidate
End Sub

'Initializes the graph
Private Sub InitGraph
	XGridNb = 10
	YGridNb = 10
	
	GridX0 = 40dip
	Grid_dX = Floor((cWidth - GridX0 - 10dip) / XGridNb)
	GridW = Grid_dX * XGridNb
	GridX1 = GridX0 + GridW
	
	GridY0 = 20dip
	Grid_dY = Floor((cHeight - GridY0 - 20dip) / YGridNb)
	GridH = Grid_dY * YGridNb
	GridY1 = GridY0 + GridH
	
	pnlGraph.Color = Colors.White
	
	pnlGrid.Initialize("pnlGrid")
	pnlGraph.AddView(pnlGrid, 0, 0, pnlGraph.Width, pnlGraph.Height)
	pnlGrid.Color = Colors.Transparent
	
	pnlCurves.Initialize("pnlCurves")
	pnlGraph.AddView(pnlCurves, GridX0, GridY0, GridW, GridH)
	rectCurves.Initialize(0, 0, pnlGraph.Width, pnlGraph.Height)
	pnlCurves.Color = Colors.Transparent

	cvsGrid.Initialize(pnlGrid)
	cvsCurves.Initialize(pnlCurves)
End Sub

'Shows the Plot
'XValues() = one dimension array of the X value of the curve
'YValues() = one dimension array of the Y value of the curve
Public Sub Show(XValues() As Double, YValues() As Double)
	X = XValues
	Y = YValues
	
	CalcGrid
	DrawGrid
	DrawTitle
	DrawCurveNames
	CalcScales
	DrawScales
	DrawCurve
End Sub

'Shows the Plot, initializes the chart without drawing a curve
Public Sub Show1
	CalcGrid
	DrawGrid
	DrawTitle
	DrawCurveNames
	CalcScales
	DrawScales
End Sub

'Calculates the Grid dimensions according to the Texts to display and autoscaling
Private Sub CalcGrid
	ly = GetDeviceLayoutValues
	If ScaleTextAuto = True AND ly.ApproximateScreenSize >= 4 Then
		txtScale = 1 + (ly.ApproximateScreenSize - 3.5) / 3.5 * ScaleTextRate
	Else
		txtScale = 1
	End If
	
	TextSizeTitle = Round2(TextSizeTitle0 * txtScale, 1)
	TextSizeCurve = Round2(TextSizeCurve0 * txtScale, 1)
	TextSizeScale = Round2(TextSizeScale0 * txtScale, 1)
	
	GridX0 = 40dip * txtScale
	If CurveYTitle <> "" Then
		GridX0 = GridX0 + cvsGrid.MeasureStringHeight("Aj", Typeface.DEFAULT, TextSizeCurve)
	End If
	Grid_dX = Floor((cWidth - GridX0 - 10dip * txtScale) / XGridNb)
	GridW = Grid_dX * XGridNb
	GridX1 = GridX0 + GridW
	
	GridY0 = 10dip * txtScale
	If PlotTitle <> "" Then
		GridY0 = GridY0 + cvsGrid.MeasureStringHeight("Aj", Typeface.DEFAULT, TextSizeTitle)
	End If
	Dim BottomMargin As Int
	BottomMargin = 7dip * txtScale + cvsGrid.MeasureStringHeight("0", Typeface.DEFAULT, TextSizeScale)
	If CurveXTitle <> "" Then
		BottomMargin = BottomMargin + cvsGrid.MeasureStringHeight("Aj", Typeface.DEFAULT, TextSizeCurve)
	End If
	Grid_dY = Floor((cHeight - GridY0 - BottomMargin) / YGridNb)
	GridH = Grid_dY * YGridNb
	GridY1 = GridY0 + GridH	
	
	pnlCurves.SetLayout(GridX0, GridY0, GridW, GridH)
	cvsCurves.Initialize(pnlCurves)

End Sub

'Draws the Grid
Private Sub DrawGrid
	Dim x1, y1 As Float
	
	For i = 0 To XGridNb
		x1 = GridX0 + i * Grid_dX
		cvsGrid.DrawLine(x1, GridY0, x1, GridY1, GridColor, GridStroke) 
	Next
	
	For i = 0 To YGridNb
		y1 = GridY0 + i * Grid_dY
		cvsGrid.DrawLine(GridX0, y1, GridX1, y1, GridColor, GridStroke) 
	Next
	pnlGrid.Invalidate
End Sub

'Calculates the scales
Private Sub CalcScales
	Dim i As Int
	
	XMin = 1e15
	XMax = -1e15
	YMin = 1e15
	YMax = -1e15
	For i = 1 To X.Length - 1
		XMin = Min(XMin, X(i))
		XMax = Max(XMax, X(i))
		YMin = Min(YMin, Y(i))
		YMax = Max(YMax, Y(i))
	Next
	Scale_dX = (ScaleXMax - ScaleXMin) / XGridNb
'	ScaleX = GridW / (ScaleXMax - ScaleXMin)
	ScaleX = GridW / SamplesNumberMax
	Scale_dY = (ScaleYMax - ScaleYMin) / YGridNb
	ScaleY = GridH / (ScaleYMax - ScaleYMin)
End Sub

'Draws the X and Y scales
Private Sub DrawScales
	Dim i As Int
	Dim x1, y1, y2 As Float
	Dim txt As String
	
	y1 = GridY1 + cvsGrid.MeasureStringHeight("A", Typeface.DEFAULT, TextSizeCurve) + 3dip * txtScale
	For i = 0 To XGridNb
		x1 = GridX0 + Grid_dX * i
		txt = NumberFormat2(ScaleXMin + i * Scale_dX, 1, 3, 0, False)
		cvsGrid.DrawText(txt, x1, y1, Typeface.DEFAULT, TextSizeScale, colCurveX, "CENTER")
	Next
	
	x1 = GridX0 - 2dip * txtScale
	y2 = cvsGrid.MeasureStringHeight("A", Typeface.DEFAULT, TextSizeCurve) / 2
	For i = 0 To YGridNb
		y1 = GridY1 - i * Grid_dY + y2
		txt = NumberFormat2(ScaleYMin + i * Scale_dY, 1, 3, 0, False)
		cvsGrid.DrawText(txt, x1, y1, Typeface.DEFAULT, TextSizeScale, colCurveY, "RIGHT")
	Next
	
End Sub

'Draws the diagram title
Private Sub DrawTitle
	If PlotTitle = "" Then Return
	
	Dim x0, y0 As Float

	x0 = pnlGraph.Width / 2
	y0 = cvsGrid.MeasureStringHeight("A", Typeface.DEFAULT, TextSizeTitle) + 6dip * txtScale
	cvsGrid.DrawText(PlotTitle, x0, y0, Typeface.DEFAULT, TextSizeTitle, colTitleText, "CENTER")
End Sub

'Draws the curve names
Sub DrawCurveNames
	If CurveXTitle <> "" Then
		Dim x0, y0 As Float
		x0 = GridX0 + GridW / 2
		y0 = pnlGraph.Height - 5dip * txtScale
		cvsGrid.DrawText(CurveXTitle, x0, y0, Typeface.DEFAULT, TextSizeCurve, colCurveX, "CENTER")
	End If
	
	If CurveYTitle <> "" Then
		Dim x0, y0 As Float
		x0 = cvsGrid.MeasureStringHeight("A", Typeface.DEFAULT, TextSizeTitle) + 3dip * txtScale
		y0 = GridY0 + GridH / 2
		cvsGrid.DrawTextRotated(CurveYTitle, x0, y0, Typeface.DEFAULT, TextSizeCurve, colCurveY, "CENTER", -90)
	End If
End Sub

'Draws the curve
Private Sub DrawCurve
	Dim x1, y1, x2, y2 As Float
	Dim i As Int
	
	x1 = (X(0) - ScaleXMin) * ScaleX
	y1 = (ScaleYMax - Y(0)) * ScaleY
	For i = 1 To x.Length - 1
		x2 = (X(i) - ScaleXMin) * ScaleX
		y2 = (ScaleYMax - Y(i)) * ScaleY
		cvsCurves.DrawLine(x1, y1, x2, y2, colCurveY, StrokeCurve)
		x1 = x2
		y1 = y2
	Next
End Sub

'Sets or gets the max X scale 
Sub setXScaleMax(XScaleMax As Double)
	ScaleXMax = XScaleMax
End Sub

Sub getXScaleMax As Double
	Return ScaleXMax
End Sub

'Sets the max and min scales
Sub SetScales(XScaleMin As Double, XScaleMax As Double, YScaleMin As Double, YScaleMax As Double)
	ScaleXMax = XScaleMax
	ScaleXMin = XScaleMin
	ScaleYMax = YScaleMax
	ScaleYMin = YScaleMin
End Sub

'Sets or gets the min X scale 
Sub setXScaleMin(XScaleMin As Double)
	ScaleXMin = XScaleMin
End Sub

Sub getXScaleMin As Double
	Return ScaleXMin
End Sub

'Sets or gets the max Y scale 
Sub setYScaleMax(YScaleMax As Double)
	ScaleYMax = YScaleMax
End Sub

Sub getYScaleMax As Double
	Return ScaleYMax
End Sub

'Sets or gets the min Y scale
Sub setYScaleMin(YScaleMin As Double)
	ScaleYMin = YScaleMin
End Sub

Sub getYScaleMin As Double
	Return ScaleYMin
End Sub

'Sets or gets the X scale and text color
'Default value Colors.Black
Sub setCurveXColor(col As Int)
	colCurveX = col
End Sub

Sub getCurveXColor As Int
	Return colCurveX
End Sub

'Sets or gets the curve Y color
'Default value = Colors.Blue
Sub setCurveYColor(col As Int)
	colCurveY = col
End Sub

Sub getCurveYColor As Int
	Return colCurveY
End Sub
'Sets or gets the curve stroke
'Default value 2dip
Sub setCurveStroke(Stroke As Float)
	StrokeCurve = Stroke
End Sub

Sub getCurveStroke As Float
	Return StrokeCurve
End Sub

'Sets the diagram title
Sub setTitle(Title As String)
	PlotTitle = Title
End Sub

Sub getTitle As String
	Return PlotTitle
End Sub

'Sets the title color
'Default value Colors.Black
Sub setTitleTextColor(col As Int)
	colTitleText = col
End Sub

Sub getTitleTextColor As Int
	Return colTitleText
End Sub

'Sets the title text size
'Default value 16
Sub setTitleTextSize(Size As Float)
	TextSizeTitle = Size
End Sub

Sub getTitleTextSize As String
	Return TextSizeTitle
End Sub

'Sets the X curve name
Sub setCurveXName(Name As String)
	CurveXTitle = Name
End Sub

Sub getCurveXName As String
	Return CurveXTitle
End Sub

'Sets the Y curve name
Sub setCurveYName(Name As String)
	CurveYTitle = Name
End Sub

Sub getCurveYName As String
	Return CurveYTitle
End Sub

'Sets or gets the Text autoscale mode
'Default = True
Sub setTextAutoScale(IsSet As Boolean)
	ScaleTextAuto = IsSet
End Sub

Sub getTextAutoScale As Boolean
	Return ScaleTextAuto
End Sub

'Sets or gets the Text autoscale rate
'Default value = 0.3
Sub setTextAutoScaleRate(Rate As Float)
	ScaleTextRate = Rate
End Sub

Sub getTextAutoScaleRate As Float
	Return ScaleTextRate
End Sub

'Sets or gets the number of X divisions
Sub setNumberXDivisions(nb As Int)
	XGridNb = nb
End Sub

Sub getNumberXDivisions As Int
	Return XGridNb
End Sub

'Sets or gets the number of Y divisions
Sub setNumberYDivisions(nb As Int)
	YGridNb = nb
End Sub

Sub getNumberYDivisions As Int
	Return YGridNb
End Sub